import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_mvvm/base/FlutterView.dart';
import 'package:flutter_mvvm/base/FlutterViewModel.dart';
import 'package:fluttermvvm/NextView.dart';

import 'HomeViewModel.dart';
import 'model/Todo.dart';

class HomeView extends FlutterView {
  @override
  FlutterViewState<FlutterView, FlutterViewModel> getViewState() {
    return _HomeViewState();
  }
}

class _HomeViewState extends FlutterViewState<HomeView, HomeViewModel> {
  @override
  void onViewModelReady() {
    debugPrint('-------------_HomeViewState onCreate');
    viewModel.getTodoList();
  }

  @override
  void onResume() {
    super.onResume();
    debugPrint('-------------_HomeViewState onResume');
  }

  @override
  void onPause() {
    super.onPause();
    debugPrint('-------------_HomeViewState onPause');
  }

  @override
  Widget getView() {
    return Scaffold(
      backgroundColor: Colors.grey[300],
      appBar: AppBar(
        title: Text('MVVM Example'),
      ),
      body: SafeArea(
        child: Column(
          children: <Widget>[
            Expanded(
                child: viewModel.isLoading
                    ? Center(child: CupertinoActivityIndicator())
                    : viewModel.list.isEmpty
                        ? Center(child: Text('No data'))
                        : ListView.separated(
                            padding: EdgeInsets.all(16),
                            itemBuilder: (_, index) =>
                                _todoItem(viewModel.list[index]),
                            separatorBuilder: (_, __) => SizedBox(height: 16),
                            itemCount: viewModel.list.length)),
            Container(
              color: Colors.grey,
              width: double.infinity,
              height: kToolbarHeight,
              child: Row(
                children: <Widget>[
                  FlatButton(
                      onPressed: () {
                        viewModel.addTodo(Todo(
                            DateTime.now().millisecondsSinceEpoch,
                            'Hi ${DateTime.now().millisecondsSinceEpoch}'));
                      },
                      child: Text('Add Todo')),
                  FlatButton(
                      onPressed: () {
                        Navigator.of(context).push(MaterialPageRoute(
                            builder: (context) => NextView()));
                      },
                      child: Text('Next Screen'))
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  @override
  HomeViewModel getViewModel() {
    return HomeViewModel();
  }

  Widget _todoItem(Todo item) {
    return Container(
      padding: EdgeInsets.all(16),
      color: Colors.white,
      child: Row(
        children: <Widget>[
          Expanded(child: Text(item.todo)),
          SizedBox(width: 8),
          FlatButton(
              onPressed: () {
                viewModel.deleteTodo(item);
              },
              child: Text('Delete'))
        ],
      ),
    );
  }
}
