import 'dart:convert';

import 'package:flutter_mvvm/base/FlutterViewModel.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'model/Todo.dart';

class HomeViewModel extends FlutterViewModel {
  List<Todo> list = [];

  void getTodoList() async {
    final data = await _getTodos();
    list.clear();
    list.addAll(data);
    isLoading = false;
    notifyListeners();
  }

  void addTodo(Todo todo) async {
    final data = await _getTodos();
    data.add(todo);
    await _setTodos(data);
    getTodoList();
  }

  void deleteTodo(Todo todo) async {
    final data = await _getTodos();
    data.removeWhere((element) => element.id == todo.id);
    await _setTodos(data);
    getTodoList();
  }

  Future<List<Todo>> _getTodos() async {
    final sp = await SharedPreferences.getInstance();
    if (!sp.containsKey('todos') || sp.getString('todos') == null) return [];
    final rawJson = sp.getString('todos');
    final parsedJsonList = jsonDecode(rawJson) as List<dynamic>;
    return parsedJsonList.map((json) => Todo.fromJson(json)).toList();
  }

  Future<List<Todo>> _setTodos(List<Todo> data) async {
    final sp = await SharedPreferences.getInstance();
    if (data == null || data.isEmpty) {
      await sp.setString('todos', null);
      return data;
    }
    final parsedJsonList = data.map((single) => single.toJson()).toList();
    await sp.setString('todos', jsonEncode(parsedJsonList));
    return data;
  }
}
