import 'package:flutter/material.dart';
import 'package:visibility_detector/visibility_detector.dart';

import 'FlutterViewModel.dart';
import 'FlutterWidget.dart';

/// Full screen widget only
abstract class FlutterView extends FlutterWidget {
  @override
  FlutterWidgetState getWidgetState() => getViewState();

  FlutterViewState getViewState();
}

abstract class FlutterViewState<T extends FlutterView,
        V extends FlutterViewModel> extends FlutterWidgetState<T, V>
    with WidgetsBindingObserver {
  bool _isVisible = false;

  @override
  void onWidgetModelReady() {
    WidgetsBinding.instance.addObserver(this);
    onViewModelReady();
  }

  @override
  Widget getWidget() => VisibilityDetector(
      key: Key(runtimeType.toString()),
      child: getView(),
      onVisibilityChanged: (visibilityInfo) {
        _isVisible = visibilityInfo.visibleFraction > 0;
        if (_isVisible) {
          onResume();
        } else {
          onPause();
        }
      });

  @override
  V getWidgetModel() => getViewModel();

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    super.didChangeAppLifecycleState(state);
    if (state == AppLifecycleState.resumed && _isVisible) {
      onResume();
    }
  }

  @override
  void dispose() {
    WidgetsBinding.instance.removeObserver(this);
    super.dispose();
  }

  /// equal to onCreate
  void onViewModelReady();

  Widget getView();

  V getViewModel();

  /// note that onCreate of next view will be called before onResume
  void onResume() {}

  void onPause() {}
}
