import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import 'package:provider/provider.dart';

import 'FlutterWidgetModel.dart';

/// Apart widget only
abstract class FlutterWidget extends StatefulWidget {
  @override
  State<StatefulWidget> createState() {
    return getWidgetState();
  }

  FlutterWidgetState getWidgetState();
}

abstract class FlutterWidgetState<T extends FlutterWidget,
        V extends FlutterWidgetModel> extends State<T>
    with AutomaticKeepAliveClientMixin<T> {
  V viewModel;

  V _builderModel;
  bool _isViewModelReady = false;

  @override
  void initState() {
    _builderModel = getWidgetModel();
    super.initState();
    SchedulerBinding.instance.addPostFrameCallback((_) {
      setState(() {
        _isViewModelReady = true;
      });
      onWidgetModelReady();
    });
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return ChangeNotifierProvider<V>.value(
        child: Consumer<V>(builder: (context, viewModel, child) {
          this.viewModel = viewModel;
          return _isViewModelReady ? getWidget() : SizedBox.shrink();
        }),
        value: _builderModel);
  }

  @override
  bool get wantKeepAlive => shouldKeepState();

  bool shouldKeepState() => false;

  /// equal to onCreate
  void onWidgetModelReady();

  Widget getWidget();

  V getWidgetModel();
}
